var gTranslator = {};
var gPropertyTypeList = {};
var gSelectedPropertyTypeIndex = 0;
var gSelectedLevelName = UNDEFINED_OPTION;
var gSelectedCategoryName = UNDEFINED_OPTION;
var gSubdictionary = {};

/*
Entry point for every desite form, initialize your data and GUI elements here.
*/
addEventListener('desiteload', async function(event)
{    
    // Translate form's GUI based on application language.    
    gTranslator = await createTranslator('/desite.modules/dictionary.json');
    var appLanguage = await desiteAPI.getApplicationLanguage();    
    gSubdictionary = await setCurrentLanguage(gTranslator, appLanguage);
    // Connect 'onSelectionChanged' to the 'selectionChanged' signal. 'onSelectionChanged' will now be continously called on every selection change.
    await desiteAPI.selectionChanged.connect(onSelectionChanged);
    // Create a list of Revit based property types.
    gPropertyTypeList = await desiteAPI.getPropertyTypeList('geometry', 'cpRevit*', true);
    // Create Dropdown menus for Types, Levels and Categories.
    createPropertyTypeDropdownMenu();    
    await createLevelDropdownMenu();
    await createCategoryDropdownMenu();        
    // Update info GUI for currently selected element.
    await updateSelectedElementInfo();
    // Reset displayed geometry based on default filters. 
    await showElementsBasedOnFilter();
    // Create a printable list for up to 12 entries.
    var maxListEntries = 12;
    createPrintableListTable(maxListEntries);
});

/*
Creates dropdown menu options for listed property types.
*/
function createPropertyTypeDropdownMenu()
{
    var dropdown = document.getElementById('dropdownPropertyType');
    dropdown.options.length = 0;
    for (i in gPropertyTypeList)
    {
        var propertyType = gPropertyTypeList[i];
        var option = document.createElement('option');
        option.textContent = propertyType.DisplayName;
        option.value = propertyType.DisplayName;
        dropdown.appendChild(option);
    }    
}

/*
Creates dropdown menu options for available categories in the model, sorted in ascending order
*/
async function createCategoryDropdownMenu()
{
    var undefinedOption = [UNDEFINED_OPTION];
    var categoryNameList = await desiteAPI.getPropertyValues('cpRevitCategory', 'xs:string', 0 ,'geometry', false);
    categoryNameList = categoryNameList.concat(undefinedOption);
    var sortedCategoryNameList = categoryNameList.sort();
    var dropdown = document.getElementById('dropdownCategory');
    for (i in sortedCategoryNameList)
    {
        var categoryName = sortedCategoryNameList[i];
        var option = document.createElement('option');
        option.textContent = categoryName;
        option.value =  categoryName;
        dropdown.appendChild(option);
    }
}

/*
Creates dropdown menu options for available levels in the model, sorted in ascending order.
*/
async function createLevelDropdownMenu()
{            
    var undefinedOption = [UNDEFINED_OPTION];
    var levelNameList = await desiteAPI.getPropertyValues('cpRevitLevel', 'xs:string', 0, 'geometry', false);
    levelNameList = levelNameList.concat(undefinedOption);
    var sortedLevelNameList = levelNameList.sort();
    var dropdown = document.getElementById('dropdownLevel');
    for (i in sortedLevelNameList)
    {
        var levelName = sortedLevelNameList[i];
        var option = document.createElement('option');
        option.textContent = levelName;
        option.value = levelName;
        dropdown.appendChild(option);
    }
}

/*
Updates the GUI with data from the currently selected geometry element.
Currently only one element is supported.
*/
async function updateSelectedElementInfo()
{
    var name = '';
    var propertyValue = '';
    var selectedElements = await desiteAPI.getSelectedElements();
    if (selectedElements.length == 0) {
        $("#message").show();
        $("#tableSelectionAndProperties").hide();
    }else {
        $("#message").hide();
        $("#tableSelectionAndProperties").show();
        if (selectedElements.length == 1) {
            var firstSelectedElement = selectedElements[0];
            var propertyName = gPropertyTypeList[gSelectedPropertyTypeIndex].DisplayName;
            var propertyDataType = gPropertyTypeList[gSelectedPropertyTypeIndex].DataType;
            name = await desiteAPI.getPropertyValue(firstSelectedElement, 'cpName', 'xs:string');
            propertyValue = await desiteAPI.getPropertyValue(firstSelectedElement, propertyName, propertyDataType);
        }else{
            alert(`${gSubdictionary["numberOfSelectedElements"]}: ${selectedElements.length}\n${gSubdictionary["firstSteps_currentlyOnlySingleElementIsSupported"]}`);
        }
    }
    $('#inputName').val(name);
    $('#inputPropertyValue').val(propertyValue);
    
}

/*
Creates a table that can be printed to pdf via button click.
It takes the maximum number of listed elements as a parameter.
*/
async function createPrintableListTable(maxEntries)
{
    var elementList = await desiteAPI.getAllElements();
    $('#tablePrintableList').empty();
    if (elementList.length > 0)
    {
        var rowIndex = 0;
        var bgColors = ["#f7f6f6", "#ffffff"];
        var innerHtml = 
        `<tr style="background: ${bgColors[(rowIndex++)%2]}">;
        <td width="40%">Name</td>;
        <td width="30%">Level</td>;
        <td width="30%">Category</td>;
        </tr>`;
        for (i in elementList)
        {
            if(i == maxEntries)
                break;
            var element = elementList[i];
            var name = await desiteAPI.getPropertyValue(element, 'cpName', 'xs:string');
            var level = await desiteAPI.getPropertyValue(element, 'cpRevitLevel', 'xs:string');
            var category = await desiteAPI.getPropertyValue(element, 'cpRevitCategory', 'xs:string');
            innerHtml += 
            `<tr style="background: ${bgColors[(rowIndex++)%2]}"><td>${name}</td>
            <td>${(level ? level : 'n/a')}</td>
            <td>${(category ? category : 'n/a')}</td>
            </tr>`;         
        }
        $('#tablePrintableList').append(innerHtml);
    }
}

/*
Displays geometric elements based on currently selected filter options.
Geometry can be filtered by 'Category' and/or 'Level'.
*/
async function showElementsBasedOnFilter()
{        
    var elementList = await desiteAPI.getAllElements('geometry');
    if (gSelectedLevelName != UNDEFINED_OPTION)
    {
        var levelNameFilter = `"${gSelectedLevelName}"`;
        elementList = await desiteAPI.filterByProperty(elementList, 'cpRevitLevel', 'xs:string', levelNameFilter, true ,'geometry');
    }    
    if (gSelectedCategoryName != UNDEFINED_OPTION)
    {        
        var categoryNameFilter = `"${gSelectedCategoryName}"`;    
        elementList = await desiteAPI.filterByProperty(elementList, 'cpRevitCategory', 'xs:string', categoryNameFilter, true ,'geometry');
    }
    await desiteAPI.showElementsOnly(elementList);
}

async function onSelectionChanged()
{
    await updateSelectedElementInfo();    
}

async function onZoomToVisibleClicked()
{
   await desiteAPI.zoomToVisible();
}

async function onZoomToSelectedClicked()
{
   await desiteAPI.zoomToSelected();
}

function onDropdownPropertyTypeChanged()
{
    var dropdown = document.getElementById('dropdownPropertyType');
    gSelectedPropertyTypeIndex = dropdown.selectedIndex;
    updateSelectedElementInfo();
}

async function onLevelDropdownMenuChanged() 
{
    var dropdown = document.getElementById('dropdownLevel');
    var selectedIndex = dropdown.selectedIndex;
    gSelectedLevelName = dropdown.options[selectedIndex].value;
    showElementsBasedOnFilter();    
}

async function onCategoryDropdownMenuChanged()
{
    var dropdown = document.getElementById('dropdownCategory');
    var selectedIndex = dropdown.selectedIndex;
    gSelectedCategoryName = dropdown.options[selectedIndex].value;
    showElementsBasedOnFilter();
}

async function onPrintClicked()
{
    await desiteMD.setPageMargins(20,20,20,20);
    await desiteMD.setOrientation(1);   
    await desiteMD.setPaperSize(0);
    var textBlocks =
    {
        content: $('#tablePrintableList').html()
    };
    var filePath = await desiteAPI.getProjectDirectory() + '/desite.modules/samples.printTemplate.html';
    await desiteMD.createPdfByTemplate(filePath,'', textBlocks);
}